"""
Test DPI and font size functionality for visufunc and newvisufunc
"""

import unittest
import inspect


class TestFontSizeAndDPI(unittest.TestCase):
    """Test that font sizes are relative and DPI is supported"""

    def test_visufunc_has_fontsize_parameter(self):
        """Test that visufunc functions have fontsize parameter"""
        import healpy as hp
        
        # Check that main viewing functions have fontsize parameter
        functions = {
            'mollview': hp.mollview,
            'gnomview': hp.gnomview,
            'cartview': hp.cartview,
            'orthview': hp.orthview,
            'azeqview': hp.azeqview
        }
        
        for func_name, func in functions.items():
            sig = inspect.signature(func)
            self.assertIn('fontsize', sig.parameters,
                         f"{func_name} function should have fontsize parameter")
            # Check default is None
            self.assertIsNone(sig.parameters['fontsize'].default,
                             f"{func_name} fontsize parameter should default to None")

    def test_graticule_has_fontsize_parameter(self):
        """Test that graticule function has fontsize parameter"""
        import healpy as hp
        
        sig = inspect.signature(hp.graticule)
        self.assertIn('fontsize', sig.parameters,
                     "graticule function should have fontsize parameter")

    def test_projview_has_dpi_parameter(self):
        """Test that projview function has dpi parameter"""
        import healpy as hp
        
        sig = inspect.signature(hp.projview)
        self.assertIn('dpi', sig.parameters,
                     "projview function should have dpi parameter")
        # Check default is None
        self.assertIsNone(sig.parameters['dpi'].default,
                         "projview dpi parameter should default to None")

    def test_projview_dpi_documented(self):
        """Test that dpi parameter is documented in projview docstring"""
        import healpy as hp
        
        docstring = hp.projview.__doc__
        self.assertIsNotNone(docstring, "projview should have a docstring")
        # Check that dpi is mentioned in the docstring
        self.assertIn('dpi', docstring.lower(),
                     "dpi parameter should be documented in docstring")


if __name__ == '__main__':
    unittest.main()



import numpy as np
import healpy as hp
import pytest


def make_small_hole_mask(nside):
    npix = hp.nside2npix(nside)
    mask = np.zeros(npix, dtype=bool)
    theta_small, phi_small = np.deg2rad(45.0), 0.0
    pix_small = hp.ang2pix(nside, theta_small, phi_small)
    mask[pix_small] = True
    return mask, pix_small


def make_mask_with_small_and_large_holes(nside):
    mask, pix_small = make_small_hole_mask(nside)
    vec_large = hp.ang2vec(np.deg2rad(120.0), np.deg2rad(60.0))
    large_pix = hp.query_disc(nside, vec_large, np.deg2rad(10.0))
    mask[large_pix] = True
    return mask, pix_small, large_pix


def test_fill_small_holes_pixel_threshold():
    nside = 8
    mask, pix_small, large_pix = make_mask_with_small_and_large_holes(nside)
    filled = hp._masktools.fill_small_holes(mask, min_size=2)
    # The single-pixel hole should be filled
    assert not filled[pix_small]
    # The large hole should remain
    assert np.all(filled[large_pix])
    # All other pixels should be unchanged (remain False)
    assert not filled[~mask].any()


def test_fill_small_holes_area_threshold():
    nside = 8
    mask, pix_small = make_small_hole_mask(nside)
    npix = mask.size
    area_per_pix = 4 * np.pi / npix * (180 * 60 / np.pi) ** 2
    filled = hp._masktools.fill_small_holes(
        mask, min_area_arcmin2=area_per_pix * 1.1
    )
    # The hole should be filled
    assert not filled[pix_small]


def test_fill_small_holes_no_fill():
    nside = 8
    mask, pix_small, large_pix = make_mask_with_small_and_large_holes(nside)
    # Threshold too small, nothing should be filled
    filled = hp._masktools.fill_small_holes(mask, min_size=1)
    np.testing.assert_array_equal(filled, mask)


def test_dist2holes_hole_min_size():
    nside = 8
    mask, pix_small, large_pix = make_mask_with_small_and_large_holes(nside)
    # With hole_min_size=2, the single-pixel hole should be filled
    d1 = hp.dist2holes(mask, hole_min_size=2)
    # The pixel at 10 should now be treated as valid (filled)
    assert d1[pix_small] > 0  # Should not be zero (not a hole anymore)
    # The large hole should remain
    assert np.any(d1[large_pix] == 0)


def test_dist2holes_hole_min_surf_arcmin2():
    nside = 8
    mask, pix_small = make_small_hole_mask(nside)
    npix = mask.size
    # Compute area per pixel
    area_per_pix = 4 * np.pi / npix * (180 * 60 / np.pi) ** 2
    # Set threshold just above one pixel
    d2 = hp.dist2holes(mask, hole_min_surf_arcmin2=area_per_pix * 1.1)
    # The hole should be filled
    assert d2[pix_small] > 0


def test_dist2holes_no_hole_filter():
    nside = 8
    npix = hp.nside2npix(nside)
    mask = np.zeros(npix, dtype=bool)
    mask[100] = True
    d = hp.dist2holes(mask)
    assert d[100] == 0
    assert np.all(d >= 0)


def test_dist2holes_all_valid():
    nside = 8
    npix = hp.nside2npix(nside)
    mask = np.zeros(npix, dtype=bool)
    d = hp.dist2holes(mask)
    # With no holes the distances saturate at maxdist (default pi)
    assert np.allclose(d, np.pi)


def test_dist2holes_all_invalid():
    nside = 8
    npix = hp.nside2npix(nside)
    mask = np.ones(npix, dtype=bool)
    d = hp.dist2holes(mask)
    # All distances should be zero (all holes)
    assert np.all(d == 0)


def test_fill_small_holes_returns_copy_and_dtype():
    nside = 8
    mask, _, _ = make_mask_with_small_and_large_holes(nside)
    filled = hp._masktools.fill_small_holes(mask, min_size=2)
    assert filled.dtype == mask.dtype
    assert not np.shares_memory(mask, filled)


@pytest.mark.parametrize(
    "kwargs, exc",
    [
        ({"min_size": -1}, ValueError),
        ({"min_area_arcmin2": -0.5}, ValueError),
        ({"min_size": 1.5}, TypeError),
    ],
)
def test_fill_small_holes_rejects_invalid_thresholds(kwargs, exc):
    nside = 8
    mask, *_ = make_mask_with_small_and_large_holes(nside)
    with pytest.raises(exc):
        hp._masktools.fill_small_holes(mask, **kwargs)


def test_fill_small_holes_requires_healpix_length():
    nside = 8
    mask = np.zeros(hp.nside2npix(nside) + 1, dtype=bool)
    with pytest.raises(ValueError):
        hp._masktools.fill_small_holes(mask, min_size=2)


def test_dist2holes_rejects_invalid_thresholds():
    nside = 8
    mask, *_ = make_mask_with_small_and_large_holes(nside)
    with pytest.raises(ValueError):
        hp.dist2holes(mask, hole_min_size=-1)
    with pytest.raises(TypeError):
        hp.dist2holes(mask, hole_min_size=1.5)
    with pytest.raises(ValueError):
        hp.dist2holes(mask, hole_min_surf_arcmin2=-1.0)


def test_dist2holes_combined_filters():
    nside = 8
    mask, pix_small, large_pix = make_mask_with_small_and_large_holes(nside)
    # Thresholds are generous individually but together still fill the small hole
    area_per_pix = 4 * np.pi / mask.size * (180 * 60 / np.pi) ** 2
    d = hp.dist2holes(
        mask,
        hole_min_size=2,
        hole_min_surf_arcmin2=area_per_pix * 2.0,
    )
    assert d[pix_small] > 0
    assert np.any(d[large_pix] == 0)



"""
Test for orthview with half_sky=True and graticule.
This test verifies the fix for the issue where graticule() failed with 
ValueError when used with orthview(half_sky=True).
"""

import matplotlib

matplotlib.use("agg")
import unittest
import matplotlib.pyplot as plt
import numpy as np

import healpy as hp


class TestOrthviewHalfSkyGraticule(unittest.TestCase):
    """Test that graticule works correctly with orthview half_sky parameter"""

    def setUp(self):
        """Set up test fixtures"""
        self.nside = 16
        self.npix = hp.nside2npix(self.nside)
        self.test_map = np.arange(self.npix, dtype=np.double)

    def tearDown(self):
        """Clean up after tests"""
        plt.close("all")

    def test_orthview_fullsky_graticule(self):
        """Test that graticule works with full sky orthview (baseline)"""
        # This should work and serves as a baseline
        hp.orthview(self.test_map, half_sky=False, title="Full Sky Test")
        hp.graticule()
        # Test passes if no exception is raised

    def test_orthview_halfsky_graticule(self):
        """Test that graticule works with half sky orthview (bug fix)"""
        # This was failing before the fix with:
        # ValueError: array of sample points is empty
        hp.orthview(self.test_map, half_sky=True, title="Half Sky Test")
        hp.graticule()
        # Test passes if no exception is raised

    def test_orthview_halfsky_graticule_with_rotation(self):
        """Test graticule with half sky orthview and rotation"""
        hp.orthview(
            self.test_map, half_sky=True, rot=[90, 0], title="Half Sky Rotated"
        )
        hp.graticule()
        # Test passes if no exception is raised

    def test_orthview_halfsky_graticule_custom_intervals(self):
        """Test graticule with half sky orthview and custom intervals"""
        hp.orthview(self.test_map, half_sky=True, title="Half Sky Custom Graticule")
        hp.graticule(dpar=30, dmer=45)
        # Test passes if no exception is raised

    def test_orthview_halfsky_multiple_graticules(self):
        """Test multiple half sky orthviews with graticules in same figure"""
        fig = plt.figure(figsize=(12, 4))
        
        # First subplot - default
        hp.orthview(
            self.test_map, half_sky=True, fig=fig, sub=(1, 3, 1), title="Half Sky 1"
        )
        hp.graticule()
        
        # Second subplot - rotated
        hp.orthview(
            self.test_map, half_sky=True, fig=fig, sub=(1, 3, 2), rot=[90, 0],
            title="Half Sky 2"
        )
        hp.graticule()
        
        # Third subplot - different latitude
        hp.orthview(
            self.test_map, half_sky=True, fig=fig, sub=(1, 3, 3), rot=[0, 45],
            title="Half Sky 3"
        )
        hp.graticule()
        # Test passes if no exception is raised


if __name__ == "__main__":
    unittest.main()



def test_is_seq():
    import numpy as np
    from healpy.cookbook import is_seq

    assert not is_seq(None)
    assert not is_seq(1)
    assert not is_seq(1.)
    assert not is_seq(np.array(1))
    assert is_seq((1, 2, 3))
    assert is_seq([1, 2, 3])
    assert is_seq(np.array([1, 2, 3]))
    assert is_seq(np.array([[1], [2], [3]]))
    assert is_seq(())
    assert is_seq([])
    assert is_seq(np.array([]))


def test_is_seq_of_seq():
    import numpy as np
    from healpy.cookbook import is_seq_of_seq

    assert not is_seq_of_seq(None)
    assert not is_seq_of_seq(1)
    assert not is_seq_of_seq(1.)
    assert not is_seq_of_seq(np.array(1))
    assert not is_seq_of_seq((1, 2, 3))
    assert not is_seq_of_seq([1, 2, 3])
    assert not is_seq_of_seq(np.array([1, 2, 3]))
    assert is_seq_of_seq(((1, 2, 3), (4, 5), (6,)))
    assert is_seq_of_seq([[1], [2, 3], [4, 5, 6]])
    assert is_seq_of_seq(np.array([[1], [2], [3]]))
    assert is_seq_of_seq(((1,), [2], np.array([3])))
    assert is_seq_of_seq(())
    assert is_seq_of_seq([])
    assert is_seq_of_seq(np.array([]))

    # allow None
    assert not is_seq_of_seq([[1], [2], None], False)
    assert is_seq_of_seq([[1], [2], None], True)


def test_len_array_or_arrays():
    import numpy as np
    from healpy.cookbook import len_array_or_arrays
    
    # Test with single array
    assert len_array_or_arrays(np.array([1, 2, 3])) == 3
    assert len_array_or_arrays([1, 2, 3, 4]) == 4
    
    # Test with list of arrays (all non-None)
    assert len_array_or_arrays([[1, 2, 3], [4, 5, 6]]) == 3
    assert len_array_or_arrays([np.array([1, 2, 3]), np.array([4, 5, 6])]) == 3
    
    # Test with None in list of arrays (regression test for bug)
    assert len_array_or_arrays([[1, 2, 3], [4, 5, 6], None]) == 3
    assert len_array_or_arrays([None, [1, 2, 3], [4, 5, 6]]) == 3
    assert len_array_or_arrays([[1, 2, 3], None, [4, 5, 6]]) == 3
    
    # Test with numpy arrays and None
    c_ee = np.linspace(0, 3e-6, 10000)
    c_ne = np.linspace(0, 1e-6, 10000)
    c_nn = np.linspace(0, 3e-5, 10000)
    c_ell = [c_nn, c_ne, c_ee, None]
    assert len_array_or_arrays(c_ell) == 10000
    
    # Test edge case: all None
    assert len_array_or_arrays([None, None, None]) == 3



import healpy as hp
from healpy.newvisufunc import projview, newprojplot
import matplotlib.pyplot as plt
import numpy as np
import pytest
import os.path

path = os.path.dirname(os.path.realpath(__file__))


@pytest.fixture
def map_data():
    return hp.read_map(
        os.path.join(
            path,
            "data",
            "wmap_band_iqumap_r9_7yr_W_v4_udgraded32_masked_smoothed10deg_fortran.fits",
        )
    )


def test_projview_mollweide(map_data):
    projview(map_data, coord=["G"], flip="astro", projection_type="mollweide")


def test_projview_mollweide_graticule(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        projection_type="mollweide",
    )


def test_projview_mollweide_graticule_labels(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="horizontal",
        projection_type="mollweide",
    )


def test_projview_mollweide_vertical_cbar(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        projection_type="mollweide",
    )


def test_projview_mollweide_extended_cbar(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=45,
        projection_type="mollweide",
        title="Mollweide projection, astro convention (default)",
    )
    newprojplot(
        theta=np.radians(50), phi=np.radians(60), marker="o", color="r", markersize=10
    )


def test_projview_mollweide_geo_convention(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=45,
        projection_type="mollweide",
        title="Mollweide projection, geo convention",
        flip="geo",
        phi_convention="clockwise",
    )
    newprojplot(
        theta=np.radians(50), phi=np.radians(60), marker="o", color="r", markersize=10
    )


def test_projview_hammer(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="hammer",
        title="Hammer projection",
    )


def test_projview_aitoff(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="aitoff",
        title="Aitoff projection",
    )


def test_projview_cart(map_data):
    projview(map_data, coord=["G"], projection_type="cart")


def test_projview_cart_labels(map_data):
    projview(
        map_data, coord=["G"], projection_type="cart", xlabel="xlabel", ylabel="ylabel"
    )


def test_projview_cart_graticule(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="horizontal",
        projection_type="cart",
        title="Cart projection",
    )


def test_projview_cart_graticule_vertical_cbar(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        projection_type="cart",
    )

def test_projview_cart_lonra_latra(map_data):
    projview(
        map_data,
        coord=["G"],
        projection_type="cart",
        lonra=[-30, 30],
        latra=[-30, 30],
)

def test_projview_lonra_latra_error(map_data):
    # Test that both lonra and latra raise error for mollweide (default projection)
    with pytest.raises(ValueError):
        projview(
            map_data,
            coord=["G"],
            lonra=[-30, 30],
            latra=[-30, 30],
    )

def test_projview_lambert_lonra_error(map_data):
    # Test that lonra raises error for lambert projection
    with pytest.raises(ValueError, match="lonra cannot be set for projection_type='lambert'"):
        projview(
            map_data,
            coord=["G"],
            projection_type="lambert",
            lonra=[-30, 30],
        )

def test_projview_lambert_half_sky(map_data):
    # Test lambert projection with half sky using latra
    projview(
        map_data,
        coord=["G"],
        projection_type="lambert",
        latra=[0, 90],
        graticule=True,
        graticule_labels=True,
        title="Lambert half-sky projection",
    )



def test_projview_3d(map_data):
    projview(
        map_data,
        coord=["G"],
        hold=False,
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="horizontal",
        projection_type="3d",
        title="3D projection",
    )


def test_projview_3d_vertical_cbar(map_data):
    projview(
        map_data,
        coord=["G"],
        hold=False,
        graticule=True,
        graticule_labels=True,
        projection_type="3d",
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        cmap="viridis",
    )


def test_projview_polar(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        cb_orientation="horizontal",
        projection_type="polar",
        title="Polar projection",
    )


def test_projview_polar_vertical_cbar(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        cb_orientation="vertical",
        projection_type="polar",
    )


def test_projview_polar_override(map_data):
    projview(
        map_data,
        coord=["G"],
        hold=False,
        graticule=True,
        graticule_labels=True,
        flip="astro",
        projection_type="polar",
        unit="cbar label",
        cb_orientation="horizontal",
        override_plot_properties={
            "cbar_shrink": 0.5,
            "cbar_pad": 0.02,
            "cbar_label_pad": -35,
            "figure_width": 16,
            "figure_size_ratio": 0.63,
        },
    )


def test_projview_hammer_fontsize(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit=r"cbar label $\alpha$",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="hammer",
        title="Hammer projection",
        fontsize={
            "xlabel": 20,
            "ylabel": 20,
            "xtick_label": 20,
            "ytick_label": 20,
            "title": 20,
            "cbar_label": 20,
            "cbar_tick_label": 20,
        },
        xtick_label_color="r",
        ytick_label_color="g",
        graticule_color="black",
    )


def test_projview_hammer_no_phi_tick_label_shift(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit=r"cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="hammer",
        title="Hammer projection",
        phi_convention="symmetrical",
    )
    s = 500
    plt.scatter(np.deg2rad(0), np.deg2rad(0), color="r", marker="x", linewidth=10, s=s)
    plt.scatter(
        np.deg2rad(120), np.deg2rad(0), color="r", marker="x", linewidth=10, s=s
    )
    plt.scatter(
        np.deg2rad(-120), np.deg2rad(0), color="r", marker="x", linewidth=10, s=s
    )
    plt.scatter(np.deg2rad(0), np.deg2rad(60), color="r", marker="x", linewidth=10, s=s)
    plt.scatter(
        np.deg2rad(0), np.deg2rad(-60), color="r", marker="x", linewidth=10, s=s
    )


def test_projview_hammer_override_axis_tick_labels(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit=r"cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="hammer",
        title="Hammer projection",
        custom_xtick_labels=["A", "B", "C", "D", "E"],
        custom_ytick_labels=["F", "G", "H", "I", "J"],
    )


def test_projview_hammer_equatorial(map_data):
    projview(
        map_data,
        coord=["G", "C"],
        graticule=True,
        graticule_labels=True,
        unit=r"cbar label",
        xlabel="RA",
        ylabel="DEC",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="hammer",
        title="Hammer projection",
    )


def test_projview_cart_local_azimuth_counterclockwise(map_data):
    local_sidereal_time = 18
    altitude = -35.206667
    rotAngles = [(180 + (local_sidereal_time * 15)) % 360, -(altitude - 90)]
    projview(
        map_data,
        coord=["G", "C"],
        graticule=True,
        graticule_labels=True,
        unit=r"cbar label",
        xlabel="azimuth",
        ylabel="zenith",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="cart",
        title="Cartesian projection",
        rot=rotAngles,
        fontsize={"xtick_label": 20},
        phi_convention="counterclockwise",
    )


def test_projview_cart_local_azimuth_clockwise(map_data):
    local_sidereal_time = 18
    altitude = -35.206667
    rotAngles = [(180 + (local_sidereal_time * 15)) % 360, -(altitude - 90)]
    projview(
        map_data,
        coord=["G", "C"],
        graticule=True,
        graticule_labels=True,
        unit=r"cbar label",
        xlabel="azimuth",
        ylabel="zenith",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="cart",
        title="Cartesian projection",
        rot=rotAngles,
        fontsize={"xtick_label": 20},
        phi_convention="clockwise",
    )


def test_projview_cart_local_azimuth_symmetrical(map_data):
    local_sidereal_time = 18
    altitude = -35.206667
    rotAngles = [(180 + (local_sidereal_time * 15)) % 360, -(altitude - 90)]
    projview(
        map_data,
        coord=["G", "C"],
        graticule=True,
        graticule_labels=True,
        unit=r"cbar label",
        xlabel="azimuth",
        ylabel="zenith",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=30,
        projection_type="cart",
        title="Cartesian projection",
        rot=rotAngles,
        fontsize={"xtick_label": 20},
        phi_convention="symmetrical",
    )


def test_projview_return_only_data(map_data):
    longitude, latitude, grid_map = projview(
        map_data, coord=["G"], return_only_data=True
    )


def test_projview_planck_colormap(map_data):
    m_scaled = map_data * 3000
    projview(
        m_scaled,
        title="Planck colormap",
        cmap="planck",
        rlabel=r"A$_{\mathsf{ex. 1}}$",
        llabel=r"$Q$",
        unit=r"$\mu$K",
        fontname="serif",
        width=10,
        show_tickmarkers=True,
        cbar_ticks=[-300, 0, 300],
        cb_orientation="vertical",
        sub=121,
        override_plot_properties={"cbar_tick_direction": "in"},
    )
    projview(
        m_scaled,
        title="Planck logarithmic colormap",
        cmap="planck_log",
        norm="symlog2",
        rlabel=r"A$_{\mathrm{ex. 3}}$",
        llabel=r"$I$",
        unit=r"$\mu$K",
        min=-1e3,
        max=1e7,
        cb_orientation="vertical",
        sub=122,
    )
    plt.tight_layout()


def test_projview_symlog_normalization(map_data):
    m_scaled = map_data * 3000
    projview(
        m_scaled,
        title="symlog normalization",
        cmap="planck",
        norm="symlog",
        rlabel=r"A$_{\mathsf{ex. 2}}$",
        llabel=r"$Q$",
        unit=r"$\mu$K",
        cbar_ticks=[-3000, -30, 0, 30, 3000],
        remove_mono=True,
        show_tickmarkers=True,
        sub=121,
        override_plot_properties={"cbar_tick_direction": "in"},
        norm_dict={"linscale": 0.5},
    )
    projview(
        m_scaled,
        title="WMAP colormap",
        cmap="wmap",
        rlabel=r"A$_{\mathrm{ex. 4}}$",
        llabel=r"$I$",
        unit=r"$\mu$K",
        fontname="serif",
        min=-100,
        max=100,
        sub=122,
    )
    plt.tight_layout()


    def test_projview_cart_bgcolor(map_data):
        projview(
            map_data,
            coord=["G"],
            graticule=True,
            graticule_labels=True,
            unit="cbar label",
            xlabel="longitude",
            ylabel="latitude",
            cb_orientation="horizontal",
            projection_type="cart",
            title="Cart projection with bgcolor",
            bgcolor="lightblue",
        )


def test_projview_no_colorbar(map_data):
    """Test that colorbar can be hidden with cbar=False"""
    projview(
        map_data,
        coord=["G"],
        cbar=False,
        projection_type="mollweide",
        title="Map without colorbar",
    )
    # Verify no colorbar was created by checking figure
    fig = plt.gcf()
    # A figure with colorbar has more than 1 axes
    # Without colorbar, should have exactly 1 axes
    assert len(fig.axes) == 1
    plt.close('all')


def test_projview_none_map_with_cbar():
    """Test that m=None works correctly even with cbar=True (default)"""
    # This should not raise an error even though cbar defaults to True
    projview(
        m=None,
        coord=["G"],
        projection_type="mollweide",
        title="Blank map for overplotting",
    )
    # Should complete without error
    plt.close('all')


def test_projview_none_map_explicit_cbar_false():
    """Test that m=None with explicit cbar=False works"""
    projview(
        m=None,
        coord=["G"],
        cbar=False,
        projection_type="mollweide",
        title="Blank map with explicit cbar=False",
    )
    plt.close('all')



def test_projview_mollweide_badcolor(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="vertical",
        min=-0.05,
        max=0.05,
        latitude_grid_spacing=45,
        projection_type="mollweide",
        title="Mollweide projection with badcolor",
        badcolor="red",
    )


def test_projview_cart_bgcolor(map_data):
    projview(
        map_data,
        coord=["G"],
        graticule=True,
        graticule_labels=True,
        unit="cbar label",
        xlabel="longitude",
        ylabel="latitude",
        cb_orientation="horizontal",
        projection_type="cart",
        title="Cart projection with bgcolor",
        bgcolor="lightblue",
    )


def test_projview_colormap_object_preservation(map_data):
    """Test that projview preserves user-modified Colormap object colors"""
    # Create a colormap with custom bad/under colors
    colormap = plt.get_cmap('viridis').copy()
    colormap.set_bad('white')
    colormap.set_under('yellow')
    
    # Call projview with the modified colormap
    projview(
        map_data,
        cmap=colormap,
        projection_type='hammer'
    )
    
    # Get the colormap from the plot
    ax = plt.gca()
    if hasattr(ax, 'collections') and len(ax.collections) > 0:
        plot_cmap = ax.collections[0].get_cmap()
        # Verify colors are preserved
        bad_is_white = np.allclose(plot_cmap._rgba_bad[:3], [1.0, 1.0, 1.0])
        under_is_yellow = np.allclose(plot_cmap._rgba_under[:3], [1.0, 1.0, 0.0])
        assert bad_is_white, f"projview should preserve Colormap bad color, got {plot_cmap._rgba_bad}"
        assert under_is_yellow, f"projview should preserve Colormap under color, got {plot_cmap._rgba_under}"



#
#  This file is part of Healpy.
#
#  Healpy is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  Healpy is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Healpy; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#  For more information about Healpy, see http://code.google.com/p/healpy
#

"""
=====================================================
visufunc.py : Healpix visualization functions
=====================================================

This module provides functions to display Healpix map.

Map projections
---------------

- :func:`mollview` displays a map using Mollweide projection (full sky)
- :func:`gnomview` displays a map using Gnomonic projection (local map)
- :func:`cartview` displays a map using Cartesian projection
- :func:`orthview` displays a map using Orthographic projection (full or half sky)
- :func:`azeqview` displays a map using Azimuthal equidistant projection

Graticules
----------

- :func:`graticule` adds a graticule to the current map
- :func:`delgraticules` deletes all graticules of a map

Tracing lines or points
-----------------------

- :func:`projplot` plots data points on the current map
- :func:`projscatter` displays scatter points
- :func:`projtext` display a text on the current map
"""

__all__ = [
    "mollview",
    "gnomview",
    "cartview",
    "orthview",
    "azeqview",
    "graticule",
    "delgraticules",
    "projplot",
    "projscatter",
    "projtext",
]

from . import projaxes as PA
import numpy as np
from . import pixelfunc
import matplotlib
import matplotlib.pyplot as plt

pi = np.pi
dtor = pi / 180.0


def mollview(
    map=None,
    fig=None,
    rot=None,
    coord=None,
    unit="",
    xsize=800,
    title="Mollweide view",
    nest=False,
    min=None,
    max=None,
    flip="astro",
    remove_dip=False,
    remove_mono=False,
    gal_cut=0,
    format="%g",
    format2="%g",
    cbar=True,
    cmap=None,
    badcolor="gray",
    bgcolor="white",
    notext=False,
    norm=None,
    hold=False,
    reuse_axes=False,
    margins=None,
    sub=None,
    nlocs=2,
    return_projected_map=False,
    alpha=None,
    fontsize=None,
):
    """Plot a healpix map (given as an array) in Mollweide projection.

    Parameters
    ----------
    map : float, array-like or None
      An array containing the map, supports masked maps, see the `ma` function.
      If None, will display a blank map, useful for overplotting.
    fig : int or None, optional
      The figure number to use. Default: create a new figure
    rot : scalar or sequence, optional
      Describe the rotation to apply.
      In the form (lon, lat, psi) (unit: degrees) : the point at
      longitude *lon* and latitude *lat* will be at the center. An additional rotation
      of angle *psi* around this direction is applied.
    coord : sequence of character, optional
      Either one of 'G', 'E' or 'C' (where 'E' stands for the Ecliptic, 'G' for the Galactic,
      and 'C' for the Celestial or equatorial) to describe the coordinate system of the map,
      or a sequence of 2 of these to rotate the map from the first to the second coordinate system.
    unit : str, optional
      A text describing the unit of the data. Default: ''
    xsize : int, optional
      The size of the image. Default: 800
    title : str, optional
      The title of the plot. Default: 'Mollweide view'
    nest : bool, optional
      If True, ordering scheme is NESTED. Default: False (RING)
    min : float, optional
      The minimum range value
    max : float, optional
      The maximum range value
    flip : {'astro', 'geo'}, optional
      Defines the convention of projection : 'astro' (default, east towards left, west towards right)
      or 'geo' (east towards right, west towards left)
    remove_dip : bool, optional
      If :const:`True`, remove the dipole+monopole
    remove_mono : bool, optional
      If :const:`True`, remove the monopole
    gal_cut : float, scalar, optional
      Symmetric galactic cut for the dipole/monopole fit.
      Removes points in latitude range [-gal_cut, +gal_cut]
    format : str, optional
      The format of the scale label. Default: '%g'
    format2 : str, optional
      Format of the pixel value under mouse. Default: '%g'
    cbar : bool, optional
      Display the colorbar. Default: True
    notext : bool, optional
      If True, no text is printed around the map
    norm : {'hist', 'log', None}
      Color normalization, hist= histogram equalized color mapping,
      log= logarithmic color mapping, default: None (linear color mapping)
    cmap : a color map
       The colormap to use (see matplotlib.cm)
    badcolor : str
      Color to use to plot bad values
    bgcolor : str
      Color to use for background
    hold : bool, optional
      If True, replace the current Axes by a MollweideAxes.
      use this if you want to have multiple maps on the same
      figure. Default: False
    sub : int, scalar or sequence, optional
      Use only a zone of the current figure (same syntax as subplot).
      Default: None
    reuse_axes : bool, optional
      If True, reuse the current Axes (should be a MollweideAxes). This is
      useful if you want to overplot with a partially transparent colormap,
      such as for plotting a line integral convolution. Default: False
    margins : None or sequence, optional
      Either None, or a sequence (left,bottom,right,top)
      giving the margins on left,bottom,right and top
      of the axes. Values are relative to figure (0-1).
      Default: None
    return_projected_map : bool
      if True returns the projected map in a 2d numpy array
    alpha : float, array-like or None
      An array containing the alpha channel, supports masked maps, see the `ma` function.
      If None, no transparency will be applied.
    fontsize : dict, optional
      Override font sizes. Allowed keys are 'xlabel', 'ylabel', 'title', 'xtick_label', 
      'ytick_label', 'cbar_label', 'cbar_tick_label'. Default: None uses relative sizes
      that scale with DPI ('large' for most text). To exactly reproduce plots from 
      previous versions, use: ``fontsize={'xlabel': 14, 'ylabel': 14, 'title': 14, 
      'cbar_label': 14}``

    See Also
    --------
    gnomview, cartview, orthview, azeqview
    """
    # Create the figure
    import pylab

    if map is None:
        map = np.zeros(12) + np.inf
        cbar = False

    # Ensure that the nside is valid
    nside = pixelfunc.get_nside(map)
    pixelfunc.check_nside(nside, nest=nest)

    # Set up font sizes - use provided values or defaults
    if fontsize is None:
        fontsize = {}
    fontsize_defaults = {
        'xlabel': 'large',
        'ylabel': 'large', 
        'title': 'large',
        'xtick_label': 'large',
        'ytick_label': 'large',
        'cbar_label': 'large',
        'cbar_tick_label': 'large',
    }
    for key in fontsize_defaults:
        if key not in fontsize:
            fontsize[key] = fontsize_defaults[key]

    if not (hold or sub or reuse_axes):
        f = pylab.figure(fig, figsize=(8.5, 5.4))
        if not margins:
            margins = (0.02, 0.05, 0.02, 0.05)
        extent = (0.0, 0.0, 1.0, 1.0)
    elif hold:
        f = pylab.gcf()
        left, bottom, right, top = np.array(f.gca().get_position()).ravel()
        if not margins:
            margins = (0.0, 0.0, 0.0, 0.0)
        extent = (left, bottom, right - left, top - bottom)
        f.delaxes(f.gca())
    elif reuse_axes:
        f = pylab.gcf()
    else:  # using subplot syntax
        f = pylab.gcf()
        if hasattr(sub, "__len__"):
            nrows, ncols, idx = sub
        else:
            nrows, ncols, idx = sub // 100, (sub % 100) // 10, (sub % 10)
        if idx < 1 or idx > ncols * nrows:
            raise ValueError("Wrong values for sub: %d, %d, %d" % (nrows, ncols, idx))
        c, r = (idx - 1) % ncols, (idx - 1) // ncols
        if not margins:
            margins = (0.01, 0.0, 0.0, 0.02)
        extent = (
            c * 1.0 / ncols,
            1.0 - (r + 1) * 1.0 / nrows,
            1.0 / ncols,
            1.0 / nrows,
        )
    if not reuse_axes:
        extent = (
            extent[0] + margins[0],
            extent[1] + margins[1],
            extent[2] - margins[2] - margins[0],
            extent[3] - margins[3] - margins[1],
        )

    # Starting to draw : turn interactive off
    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    try:
        map = pixelfunc.ma_to_array(map)
        if alpha is not None:
            alpha = pixelfunc.ma_to_array(alpha)
        if reuse_axes:
            ax = f.gca()
        else:
            ax = PA.HpxMollweideAxes(
                f, extent, coord=coord, rot=rot, format=format2, flipconv=flip
            )
            f.add_axes(ax)
        if remove_dip:
            map = pixelfunc.remove_dipole(
                map, gal_cut=gal_cut, nest=nest, copy=True
            )
        elif remove_mono:
            map = pixelfunc.remove_monopole(
                map, gal_cut=gal_cut, nest=nest, copy=True
            )
        img = ax.projmap(
            map,
            nest=nest,
            xsize=xsize,
            coord=coord,
            vmin=min,
            vmax=max,
            cmap=cmap,
            badcolor=badcolor,
            bgcolor=bgcolor,
            norm=norm,
            alpha=alpha,
        )
        if cbar:
            im = ax.get_images()[0]
            b = im.norm.inverse(np.linspace(0, 1, im.cmap.N + 1))
            v = np.linspace(im.norm.vmin, im.norm.vmax, im.cmap.N)
            mappable = plt.cm.ScalarMappable(
                norm=matplotlib.colors.Normalize(vmin=im.norm.vmin, vmax=im.norm.vmax),
                cmap=cmap,
            )
            if matplotlib.__version__ >= "0.91.0":
                cb = f.colorbar(
                    mappable,
                    ax=ax,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(nlocs, norm),
                    pad=0.05,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            else:
                # for older matplotlib versions, no ax kwarg
                cb = f.colorbar(
                    mappable,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(nlocs, norm),
                    pad=0.05,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            cb.solids.set_rasterized(True)
        ax.set_title(title)
        coordsys_label = ax.proj.coordsysstr
        if not notext and coordsys_label:
            ax.text(
                0.86,
                0.05,
                coordsys_label,
                fontsize=fontsize['cbar_label'],
                fontweight="bold",
                transform=ax.transAxes,
            )
        if cbar:
            cb.ax.text(
                0.5,
                -1.0,
                unit,
                fontsize=fontsize['cbar_label'],
                transform=cb.ax.transAxes,
                ha="center",
                va="center",
            )
        f.sca(ax)
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()
            # pylab.show()
    if return_projected_map:
        return img


def gnomview(
    map=None,
    fig=None,
    rot=None,
    coord=None,
    unit="",
    xsize=200,
    ysize=None,
    reso=1.5,
    title="Gnomonic view",
    nest=False,
    remove_dip=False,
    remove_mono=False,
    gal_cut=0,
    min=None,
    max=None,
    flip="astro",
    format="%.3g",
    cbar=True,
    cmap=None,
    badcolor="gray",
    bgcolor="white",
    norm=None,
    hold=False,
    sub=None,
    reuse_axes=False,
    margins=None,
    notext=False,
    return_projected_map=False,
    no_plot=False,
    alpha=None,
    fontsize=None,
):
    """Plot a healpix map (given as an array) in Gnomonic projection.

    Parameters
    ----------
    map : array-like
      The map to project, supports masked maps, see the `ma` function.
      If None, use a blank map, useful for
      overplotting.
    fig : None or int, optional
      A figure number. Default: None= create a new figure
    rot : scalar or sequence, optional
      Describe the rotation to apply.
      In the form (lon, lat, psi) (unit: degrees) : the point at
      longitude *lon* and latitude *lat* will be at the center. An additional rotation
      of angle *psi* around this direction is applied.
    coord : sequence of character, optional
      Either one of 'G', 'E' or 'C' (where 'E' stands for the Ecliptic, 'G' for the Galactic,
      and 'C' for the Celestial or equatorial) to describe the coordinate system of the map,
      or a sequence of 2 of these to rotate the map from the first to the second coordinate system.
    unit : str, optional
      A text describing the unit of the data. Default: ''
    xsize : int, optional
      The size of the image. Default: 200
    ysize : None or int, optional
      The size of the image. Default: None= xsize
    reso : float, optional
      Resolution (in arcmin). Default: 1.5 arcmin
    title : str, optional
      The title of the plot. Default: 'Gnomonic view'
    nest : bool, optional
      If True, ordering scheme is NESTED. Default: False (RING)
    min : float, scalar, optional
      The minimum range value
    max : float, scalar, optional
      The maximum range value
    flip : {'astro', 'geo'}, optional
      Defines the convention of projection : 'astro' (default, east towards left, west towards right)
      or 'geo' (east towards roght, west towards left)
    remove_dip : bool, optional
      If :const:`True`, remove the dipole+monopole
    remove_mono : bool, optional
      If :const:`True`, remove the monopole
    gal_cut : float, scalar, optional
      Symmetric galactic cut for the dipole/monopole fit.
      Removes points in latitude range [-gal_cut, +gal_cut]
    format : str, optional
      The format of the scale label. Default: '%g'
    cmap : a color map
       The colormap to use (see matplotlib.cm)
    badcolor : str
      Color to use to plot bad values
    bgcolor : str
      Color to use for background
    hold : bool, optional
      If True, replace the current Axes by a GnomonicAxes.
      use this if you want to have multiple maps on the same
      figure. Default: False
    sub : int or sequence, optional
      Use only a zone of the current figure (same syntax as subplot).
      Default: None
    reuse_axes : bool, optional
      If True, reuse the current Axes (should be a GnomonicAxes). This is
      useful if you want to overplot with a partially transparent colormap,
      such as for plotting a line integral convolution. Default: False
    margins : None or sequence, optional
      Either None, or a sequence (left,bottom,right,top)
      giving the margins on left,bottom,right and top
      of the axes. Values are relative to figure (0-1).
      Default: None
    notext: bool, optional
      If True: do not add resolution info text. Default=False
    return_projected_map : bool, optional
      if True returns the projected map in a 2d numpy array
    no_plot : bool, optional
      if True no figure will be created
    alpha : float, array-like or None
      An array containing the alpha channel, supports masked maps, see the `ma` function.
      If None, no transparency will be applied.
      See an example usage of the alpha channel transparency in the documentation under
      "Other tutorials"
    fontsize : dict, optional
      Override font sizes. Allowed keys are 'xlabel', 'ylabel', 'title', 'xtick_label', 
      'ytick_label', 'cbar_label', 'cbar_tick_label'. Default: None uses relative sizes
      that scale with DPI ('large' for most text, 'medium' for resolution info). 
      To exactly reproduce plots from previous versions, use: 
      ``fontsize={'cbar_label': 14, 'xtick_label': 12}``

    See Also
    --------
    mollview, cartview, orthview, azeqview
    """
    import pylab

    if map is None:
        map = np.zeros(12) + np.inf
        cbar = False

    # Ensure that the nside is valid
    nside = pixelfunc.get_nside(map)
    pixelfunc.check_nside(nside, nest=nest)

    # Set up font sizes - use provided values or defaults
    if fontsize is None:
        fontsize = {}
    fontsize_defaults = {
        'xlabel': 'large',
        'ylabel': 'large', 
        'title': 'large',
        'xtick_label': 'medium',  # resolution info text
        'ytick_label': 'large',
        'cbar_label': 'large',
        'cbar_tick_label': 'large',
    }
    for key in fontsize_defaults:
        if key not in fontsize:
            fontsize[key] = fontsize_defaults[key]

    if not (hold or sub or reuse_axes):
        f = pylab.figure(fig, figsize=(5.8, 6.4))
        if not margins:
            margins = (0.075, 0.05, 0.075, 0.05)
        extent = (0.0, 0.0, 1.0, 1.0)
    elif hold:
        f = pylab.gcf()
        left, bottom, right, top = np.array(pylab.gca().get_position()).ravel()
        if not margins:
            margins = (0.0, 0.0, 0.0, 0.0)
        extent = (left, bottom, right - left, top - bottom)
        f.delaxes(pylab.gca())
    elif reuse_axes:
        f = pylab.gcf()
    else:  # using subplot syntax
        f = pylab.gcf()
        if hasattr(sub, "__len__"):
            nrows, ncols, idx = sub
        else:
            nrows, ncols, idx = sub // 100, (sub % 100) // 10, (sub % 10)
        if idx < 1 or idx > ncols * nrows:
            raise ValueError("Wrong values for sub: %d, %d, %d" % (nrows, ncols, idx))
        c, r = (idx - 1) % ncols, (idx - 1) // ncols
        if not margins:
            margins = (0.01, 0.0, 0.0, 0.02)
        extent = (
            c * 1.0 / ncols,
            1.0 - (r + 1) * 1.0 / nrows,
            1.0 / ncols,
            1.0 / nrows,
        )
    if not reuse_axes:
        extent = (
            extent[0] + margins[0],
            extent[1] + margins[1],
            extent[2] - margins[2] - margins[0],
            extent[3] - margins[3] - margins[1],
        )
    # f=pylab.figure(fig,figsize=(5.5,6))

    # Starting to draw : turn interactive off
    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    try:
        map = pixelfunc.ma_to_array(map)
        if reuse_axes:
            ax = f.gca()
        else:
            ax = PA.HpxGnomonicAxes(
                f, extent, coord=coord, rot=rot, format=format, flipconv=flip
            )
            f.add_axes(ax)
        if remove_dip:
            map = pixelfunc.remove_dipole(map, gal_cut=gal_cut, nest=nest, copy=True)
        elif remove_mono:
            map = pixelfunc.remove_monopole(map, gal_cut=gal_cut, nest=nest, copy=True)
        img = ax.projmap(
            map,
            nest=nest,
            coord=coord,
            vmin=min,
            vmax=max,
            xsize=xsize,
            ysize=ysize,
            reso=reso,
            cmap=cmap,
            norm=norm,
            badcolor=badcolor,
            bgcolor=bgcolor,
            alpha=alpha,
        )
        if cbar:
            im = ax.get_images()[0]
            b = im.norm.inverse(np.linspace(0, 1, im.cmap.N + 1))
            v = np.linspace(im.norm.vmin, im.norm.vmax, im.cmap.N)
            mappable = plt.cm.ScalarMappable(
                norm=matplotlib.colors.Normalize(vmin=im.norm.vmin, vmax=im.norm.vmax),
                cmap=cmap,
            )
            if matplotlib.__version__ >= "0.91.0":
                cb = f.colorbar(
                    mappable,
                    ax=ax,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(),
                    pad=0.08,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            else:
                cb = f.colorbar(
                    mappable,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(),
                    pad=0.08,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            cb.solids.set_rasterized(True)
        ax.set_title(title)
        if not notext:
            ax.text(
                -0.07,
                0.02,
                "%g '/pix,   %dx%d pix"
                % (
                    ax.proj.arrayinfo["reso"],
                    ax.proj.arrayinfo["xsize"],
                    ax.proj.arrayinfo["ysize"],
                ),
                fontsize=fontsize['xtick_label'],
                verticalalignment="bottom",
                transform=ax.transAxes,
                rotation=90,
            )
            coordsys_label = ax.proj.coordsysstr
            if coordsys_label:
                ax.text(
                    -0.07,
                    0.6,
                    coordsys_label,
                    fontsize=fontsize['cbar_label'],
                    fontweight="bold",
                    rotation=90,
                    transform=ax.transAxes,
                )
            lon, lat = np.around(ax.proj.get_center(lonlat=True), ax._coordprec)
            ax.text(
                0.5,
                -0.03,
                "(%g,%g)" % (lon, lat),
                verticalalignment="center",
                horizontalalignment="center",
                transform=ax.transAxes,
            )
        if cbar:
            cb.ax.text(
                1.05,
                0.30,
                unit,
                fontsize=fontsize['cbar_label'],
                fontweight="bold",
                transform=cb.ax.transAxes,
                ha="left",
                va="center",
            )
        f.sca(ax)
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()
            # pylab.show()
        if no_plot:
            pylab.close(f)
            f.clf()
            ax.cla()
    if return_projected_map:
        return img


def cartview(
    map=None,
    fig=None,
    rot=None,
    zat=None,
    coord=None,
    unit="",
    xsize=800,
    ysize=None,
    lonra=None,
    latra=None,
    title="Cartesian view",
    nest=False,
    remove_dip=False,
    remove_mono=False,
    gal_cut=0,
    min=None,
    max=None,
    flip="astro",
    format="%.3g",
    cbar=True,
    cmap=None,
    badcolor="gray",
    bgcolor="white",
    norm=None,
    aspect=None,
    hold=False,
    sub=None,
    reuse_axes=False,
    margins=None,
    notext=False,
    return_projected_map=False,
    alpha=None,
    fontsize=None,
):
    """Plot a healpix map (given as an array) in Cartesian projection.

    Parameters
    ----------
    map : float, array-like or None
      An array containing the map,
      supports masked maps, see the `ma` function.
      If None, will display a blank map, useful for overplotting.
    fig : int or None, optional
      The figure number to use. Default: create a new figure
    rot : scalar or sequence, optional
      Describe the rotation to apply.
      In the form (lon, lat, psi) (unit: degrees) : the point at
      longitude *lon* and latitude *lat* will be at the center. An additional rotation
      of angle *psi* around this direction is applied.
    coord : sequence of character, optional
      Either one of 'G', 'E' or 'C' (where 'E' stands for the Ecliptic, 'G' for the Galactic,
      and 'C' for the Celestial or equatorial) to describe the coordinate system of the map,
      or a sequence of 2 of these to rotate the map from the first to the second coordinate system.
    unit : str, optional
      A text describing the unit of the data. Default: ''
    xsize : int, optional
      The size of the image. Default: 800
    lonra : sequence, optional
      Range in longitude. Default: [-180,180]
    latra : sequence, optional
      Range in latitude. Default: [-90,90]
    title : str, optional
      The title of the plot. Default: 'Mollweide view'
    nest : bool, optional
      If True, ordering scheme is NESTED. Default: False (RING)
    min : float, optional
      The minimum range value
    max : float, optional
      The maximum range value
    flip : {'astro', 'geo'}, optional
      Defines the convention of projection : 'astro' (default, east towards left, west towards right)
      or 'geo' (east towards roght, west towards left)
    remove_dip : bool, optional
      If :const:`True`, remove the dipole+monopole
    remove_mono : bool, optional
      If :const:`True`, remove the monopole
    gal_cut : float, scalar, optional
      Symmetric galactic cut for the dipole/monopole fit.
      Removes points in latitude range [-gal_cut, +gal_cut]
    format : str, optional
      The format of the scale label. Default: '%g'
    cbar : bool, optional
      Display the colorbar. Default: True
    notext : bool, optional
      If True, no text is printed around the map
    norm : {'hist', 'log', None}, optional
      Color normalization, hist= histogram equalized color mapping,
      log= logarithmic color mapping, default: None (linear color mapping)
    cmap : a color map
       The colormap to use (see matplotlib.cm)
    badcolor : str
      Color to use to plot bad values
    bgcolor : str
      Color to use for background
    hold : bool, optional
      If True, replace the current Axes by a CartesianAxes.
      use this if you want to have multiple maps on the same
      figure. Default: False
    sub : int, scalar or sequence, optional
      Use only a zone of the current figure (same syntax as subplot).
      Default: None
    reuse_axes : bool, optional
      If True, reuse the current Axes (should be a CartesianAxes). This is
      useful if you want to overplot with a partially transparent colormap,
      such as for plotting a line integral convolution. Default: False
    margins : None or sequence, optional
      Either None, or a sequence (left,bottom,right,top)
      giving the margins on left,bottom,right and top
      of the axes. Values are relative to figure (0-1).
      Default: None
    return_projected_map : bool
      if True returns the projected map in a 2d numpy array
    alpha : float, array-like or None
      An array containing the alpha channel, supports masked maps, see the `ma` function.
      If None, no transparency will be applied.
    fontsize : dict, optional
      Override font sizes. Allowed keys are 'xlabel', 'ylabel', 'title', 'xtick_label', 
      'ytick_label', 'cbar_label', 'cbar_tick_label'. Default: None uses relative sizes
      that scale with DPI ('large' for most text). To exactly reproduce plots from 
      previous versions, use: ``fontsize={'cbar_label': 14}``

    See Also
    --------
    mollview, gnomview, orthview, azeqview
    """
    import pylab

    if map is None:
        map = np.zeros(12) + np.inf
        cbar = False

    # Ensure that the nside is valid
    nside = pixelfunc.get_nside(map)
    pixelfunc.check_nside(nside, nest=nest)

    # Set up font sizes - use provided values or defaults
    if fontsize is None:
        fontsize = {}
    fontsize_defaults = {
        'xlabel': 'large',
        'ylabel': 'large', 
        'title': 'large',
        'xtick_label': 'large',
        'ytick_label': 'large',
        'cbar_label': 'large',
        'cbar_tick_label': 'large',
    }
    for key in fontsize_defaults:
        if key not in fontsize:
            fontsize[key] = fontsize_defaults[key]

    if not (hold or sub or reuse_axes):
        f = pylab.figure(fig, figsize=(8.5, 5.4))
        if not margins:
            margins = (0.075, 0.05, 0.075, 0.05)
        extent = (0.0, 0.0, 1.0, 1.0)
    elif hold:
        f = pylab.gcf()
        left, bottom, right, top = np.array(pylab.gca().get_position()).ravel()
        if not margins:
            margins = (0.0, 0.0, 0.0, 0.0)
        extent = (left, bottom, right - left, top - bottom)
        f.delaxes(pylab.gca())
    elif reuse_axes:
        f = pylab.gcf()
    else:  # using subplot syntax
        f = pylab.gcf()
        if hasattr(sub, "__len__"):
            nrows, ncols, idx = sub
        else:
            nrows, ncols, idx = sub // 100, (sub % 100) // 10, (sub % 10)
        if idx < 1 or idx > ncols * nrows:
            raise ValueError("Wrong values for sub: %d, %d, %d" % (nrows, ncols, idx))
        c, r = (idx - 1) % ncols, (idx - 1) // ncols
        if not margins:
            margins = (0.01, 0.0, 0.0, 0.02)
        extent = (
            c * 1.0 / ncols,
            1.0 - (r + 1) * 1.0 / nrows,
            1.0 / ncols,
            1.0 / nrows,
        )
    if not reuse_axes:
        extent = (
            extent[0] + margins[0],
            extent[1] + margins[1],
            extent[2] - margins[2] - margins[0],
            extent[3] - margins[3] - margins[1],
        )

    # f=pylab.figure(fig,figsize=(5.5,6))
    # Starting to draw : turn interactive off
    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    try:
        map = pixelfunc.ma_to_array(map)
        if zat and rot:
            raise ValueError("Only give rot or zat, not both")
        if zat:
            rot = np.array(zat, dtype=np.float64)
            rot.resize(3)
            rot[1] -= 90
        if reuse_axes:
            ax = f.gca()
        else:
            ax = PA.HpxCartesianAxes(
                f, extent, coord=coord, rot=rot, format=format, flipconv=flip
            )
            f.add_axes(ax)
        if remove_dip:
            map = pixelfunc.remove_dipole(map, gal_cut=gal_cut, nest=nest, copy=True)
        elif remove_mono:
            map = pixelfunc.remove_monopole(map, gal_cut=gal_cut, nest=nest, copy=True)
        img = ax.projmap(
            map,
            nest=nest,
            coord=coord,
            vmin=min,
            vmax=max,
            xsize=xsize,
            ysize=ysize,
            lonra=lonra,
            latra=latra,
            cmap=cmap,
            badcolor=badcolor,
            bgcolor=bgcolor,
            norm=norm,
            aspect=aspect,
            alpha=alpha,
        )
        if cbar:
            im = ax.get_images()[0]
            b = im.norm.inverse(np.linspace(0, 1, im.cmap.N + 1))
            v = np.linspace(im.norm.vmin, im.norm.vmax, im.cmap.N)
            mappable = plt.cm.ScalarMappable(
                norm=matplotlib.colors.Normalize(vmin=im.norm.vmin, vmax=im.norm.vmax),
                cmap=cmap,
            )
            if matplotlib.__version__ >= "0.91.0":
                cb = f.colorbar(
                    mappable,
                    ax=ax,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(),
                    pad=0.08,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            else:
                cb = f.colorbar(
                    mappable,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(),
                    pad=0.08,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            cb.solids.set_rasterized(True)
        ax.set_title(title)
        coordsys_label = ax.proj.coordsysstr
        if not notext and coordsys_label:
            ax.text(
                -0.07,
                0.6,
                coordsys_label,
                fontsize=fontsize['cbar_label'],
                fontweight="bold",
                rotation=90,
                transform=ax.transAxes,
            )
        if cbar:
            cb.ax.text(
                1.05,
                0.30,
                unit,
                fontsize=fontsize['cbar_label'],
                fontweight="bold",
                transform=cb.ax.transAxes,
                ha="left",
                va="center",
            )
        f.sca(ax)
    finally:
        if wasinteractive:
            pylab.ion()
            pylab.draw()
            # pylab.show()
    if return_projected_map:
        return img


def orthview(
    map=None,
    fig=None,
    rot=None,
    coord=None,
    unit="",
    xsize=800,
    half_sky=False,
    title="Orthographic view",
    nest=False,
    min=None,
    max=None,
    flip="astro",
    remove_dip=False,
    remove_mono=False,
    gal_cut=0,
    format="%g",
    format2="%g",
    cbar=True,
    cmap=None,
    badcolor="gray",
    bgcolor="white",
    notext=False,
    norm=None,
    hold=False,
    margins=None,
    sub=None,
    reuse_axes=False,
    return_projected_map=False,
    alpha=None,
    fontsize=None,
):
    """Plot a healpix map (given as an array) in Orthographic projection.

    Parameters
    ----------
    map : float, array-like or None
      An array containing the map.
      If None, will display a blank map, useful for overplotting.
    fig : int or None, optional
      The figure number to use. Default: create a new figure
    rot : scalar or sequence, optional
      Describe the rotation to apply.
      In the form (lon, lat, psi) (unit: degrees) : the point at
      longitude *lon* and latitude *lat* will be at the center. An additional rotation
      of angle *psi* around this direction is applied.
    coord : sequence of character, optional
      Either one of 'G', 'E' or 'C' (where 'E' stands for the Ecliptic, 'G' for the Galactic,
      and 'C' for the Celestial or equatorial) to describe the coordinate system of the map,
      or a sequence of 2 of these to rotate the map from the first to the second coordinate system.
    half_sky : bool, optional
      Plot only one side of the sphere. Default: False
    unit : str, optional
      A text describing the unit of the data. Default: ''
    xsize : int, optional
      The size of the image. Default: 800
    title : str, optional
      The title of the plot. Default: 'Orthographic view'
    nest : bool, optional
      If True, ordering scheme is NESTED. Default: False (RING)
    min : float, optional
      The minimum range value
    max : float, optional
      The maximum range value
    flip : {'astro', 'geo'}, optional
      Defines the convention of projection : 'astro' (default, east towards left, west towards right)
      or 'geo' (east towards roght, west towards left)
    remove_dip : bool, optional
      If :const:`True`, remove the dipole+monopole
    remove_mono : bool, optional
      If :const:`True`, remove the monopole
    gal_cut : float, scalar, optional
      Symmetric galactic cut for the dipole/monopole fit.
      Removes points in latitude range [-gal_cut, +gal_cut]
    format : str, optional
      The format of the scale label. Default: '%g'
    format2 : str, optional
      Format of the pixel value under mouse. Default: '%g'
    cbar : bool, optional
      Display the colorbar. Default: True
    notext : bool, optional
      If True, no text is printed around the map
    norm : {'hist', 'log', None}
      Color normalization, hist= histogram equalized color mapping,
      log= logarithmic color mapping, default: None (linear color mapping)
    cmap : a color map
       The colormap to use (see matplotlib.cm)
    badcolor : str
      Color to use to plot bad values
    bgcolor : str
      Color to use for background
    hold : bool, optional
      If True, replace the current Axes by an OrthographicAxes.
      use this if you want to have multiple maps on the same
      figure. Default: False
    sub : int, scalar or sequence, optional
      Use only a zone of the current figure (same syntax as subplot).
      Default: None
    reuse_axes : bool, optional
      If True, reuse the current Axes (should be a OrthographicAxes). This is
      useful if you want to overplot with a partially transparent colormap,
      such as for plotting a line integral convolution. Default: False
    margins : None or sequence, optional
      Either None, or a sequence (left,bottom,right,top)
      giving the margins on left,bottom,right and top
      of the axes. Values are relative to figure (0-1).
      Default: None
    return_projected_map : bool
      if True returns the projected map in a 2d numpy array
    alpha : float, array-like or None
      An array containing the alpha channel, supports masked maps, see the `ma` function.
      If None, no transparency will be applied.
    fontsize : dict, optional
      Override font sizes. Allowed keys are 'xlabel', 'ylabel', 'title', 'xtick_label', 
      'ytick_label', 'cbar_label', 'cbar_tick_label'. Default: None uses relative sizes
      that scale with DPI ('large' for most text). To exactly reproduce plots from 
      previous versions, use: ``fontsize={'cbar_label': 14}``

    See Also
    --------
    mollview, gnomview, cartview, azeqview
    """
    # Create the figure
    import pylab

    if map is None:
        map = np.zeros(12) + np.inf
        cbar = False

    # Ensure that the nside is valid
    nside = pixelfunc.get_nside(map)
    pixelfunc.check_nside(nside, nest=nest)

    # Set up font sizes - use provided values or defaults
    if fontsize is None:
        fontsize = {}
    fontsize_defaults = {
        'xlabel': 'large',
        'ylabel': 'large', 
        'title': 'large',
        'xtick_label': 'large',
        'ytick_label': 'large',
        'cbar_label': 'large',
        'cbar_tick_label': 'large',
    }
    for key in fontsize_defaults:
        if key not in fontsize:
            fontsize[key] = fontsize_defaults[key]

    if not (hold or sub or reuse_axes):
        f = pylab.figure(fig, figsize=(8.5, 5.4))
        if not margins:
            margins = (0.02, 0.05, 0.02, 0.05)
        extent = (0.0, 0.0, 1.0, 1.0)
    elif hold:
        f = pylab.gcf()
        left, bottom, right, top = np.array(f.gca().get_position()).ravel()
        if not margins:
            margins = (0.0, 0.0, 0.0, 0.0)
        extent = (left, bottom, right - left, top - bottom)
        f.delaxes(f.gca())
    elif reuse_axes:
        f = pylab.gcf()
    else:  # using subplot syntax
        f = pylab.gcf()
        if hasattr(sub, "__len__"):
            nrows, ncols, idx = sub
        else:
            nrows, ncols, idx = sub // 100, (sub % 100) // 10, (sub % 10)
        if idx < 1 or idx > ncols * nrows:
            raise ValueError("Wrong values for sub: %d, %d, %d" % (nrows, ncols, idx))
        c, r = (idx - 1) % ncols, (idx - 1) // ncols
        if not margins:
            margins = (0.01, 0.0, 0.0, 0.02)
        extent = (
            c * 1.0 / ncols,
            1.0 - (r + 1) * 1.0 / nrows,
            1.0 / ncols,
            1.0 / nrows,
        )
    if not reuse_axes:
        extent = (
            extent[0] + margins[0],
            extent[1] + margins[1],
            extent[2] - margins[2] - margins[0],
            extent[3] - margins[3] - margins[1],
        )

    # Starting to draw : turn interactive off
    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    try:
        if reuse_axes:
            ax = f.gca()
        else:
            ax = PA.HpxOrthographicAxes(
                f, extent, coord=coord, rot=rot, format=format2, flipconv=flip
            )
            f.add_axes(ax)
        if remove_dip:
            map = pixelfunc.remove_dipole(
                map, gal_cut=gal_cut, nest=nest, copy=True
            )
        elif remove_mono:
            map = pixelfunc.remove_monopole(
                map, gal_cut=gal_cut, nest=nest, copy=True
            )
        img = ax.projmap(
            map,
            nest=nest,
            xsize=xsize,
            half_sky=half_sky,
            coord=coord,
            vmin=min,
            vmax=max,
            cmap=cmap,
            badcolor=badcolor,
            bgcolor=bgcolor,
            norm=norm,
            alpha=alpha,
        )
        if cbar:
            im = ax.get_images()[0]
            b = im.norm.inverse(np.linspace(0, 1, im.cmap.N + 1))
            v = np.linspace(im.norm.vmin, im.norm.vmax, im.cmap.N)
            mappable = plt.cm.ScalarMappable(
                norm=matplotlib.colors.Normalize(vmin=im.norm.vmin, vmax=im.norm.vmax),
                cmap=cmap,
            )
            if matplotlib.__version__ >= "0.91.0":
                cb = f.colorbar(
                    mappable,
                    ax=ax,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(),
                    pad=0.05,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            else:
                # for older matplotlib versions, no ax kwarg
                cb = f.colorbar(
                    mappable,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(),
                    pad=0.05,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            cb.solids.set_rasterized(True)
        ax.set_title(title)
        coordsys_label = ax.proj.coordsysstr
        if not notext and coordsys_label:
            ax.text(
                0.86,
                0.05,
                coordsys_label,
                fontsize=fontsize['cbar_label'],
                fontweight="bold",
                transform=ax.transAxes,
            )
        if cbar:
            cb.ax.text(
                0.5,
                -1.0,
                unit,
                fontsize=fontsize['cbar_label'],
                transform=cb.ax.transAxes,
                ha="center",
                va="center",
            )
        f.sca(ax)
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()
            # pylab.show()
    if return_projected_map:
        return img


def azeqview(
    map=None,
    fig=None,
    rot=None,
    zat=None,
    coord=None,
    unit="",
    xsize=800,
    ysize=None,
    reso=1.5,
    lamb=False,
    half_sky=False,
    title=None,
    nest=False,
    remove_dip=False,
    remove_mono=False,
    gal_cut=0,
    min=None,
    max=None,
    flip="astro",
    format="%.3g",
    cbar=True,
    cmap=None,
    badcolor="gray",
    bgcolor="white",
    norm=None,
    aspect=None,
    hold=False,
    sub=None,
    reuse_axes=False,
    margins=None,
    notext=False,
    return_projected_map=False,
    alpha=None,
    fontsize=None,
):
    """Plot a healpix map (given as an array) in Azimuthal equidistant projection
    or Lambert azimuthal equal-area projection.

    Parameters
    ----------
    map : float, array-like or None
      An array containing the map,
      supports masked maps, see the `ma` function.
      If None, will display a blank map, useful for overplotting.
    fig : int or None, optional
      The figure number to use. Default: create a new figure
    rot : scalar or sequence, optional
      Describe the rotation to apply.
      In the form (lon, lat, psi) (unit: degrees) : the point at
      longitude *lon* and latitude *lat* will be at the center. An additional rotation
      of angle *psi* around this direction is applied.
    coord : sequence of character, optional
      Either one of 'G', 'E' or 'C' (where 'E' stands for the Ecliptic, 'G' for the Galactic,
      and 'C' for the Celestial or equatorial) to describe the coordinate system of the map,
      or a sequence of 2 of these to rotate the map from the first to the second coordinate system.
    unit : str, optional
      A text describing the unit of the data. Default: ''
    xsize : int, optional
      The size of the image. Default: 800
    ysize : None or int, optional
      The size of the image. Default: None= xsize
    reso : float, optional
      Resolution (in arcmin). Default: 1.5 arcmin
    lamb : bool, optional
      If True, plot Lambert azimuthal equal area instead of azimuthal
      equidistant. Default: False (az equidistant)
    half_sky : bool, optional
      Plot only one side of the sphere. Default: False
    title : str, optional
      The title of the plot. Default: 'Azimuthal equidistant view'
      or 'Lambert azimuthal equal-area view' (if lamb is True)
    nest : bool, optional
      If True, ordering scheme is NESTED. Default: False (RING)
    min : float, optional
      The minimum range value
    max : float, optional
      The maximum range value
    flip : {'astro', 'geo'}, optional
      Defines the convention of projection : 'astro' (default, east towards left, west towards right)
      or 'geo' (east towards roght, west towards left)
    remove_dip : bool, optional
      If :const:`True`, remove the dipole+monopole
    remove_mono : bool, optional
      If :const:`True`, remove the monopole
    gal_cut : float, scalar, optional
      Symmetric galactic cut for the dipole/monopole fit.
      Removes points in latitude range [-gal_cut, +gal_cut]
    format : str, optional
      The format of the scale label. Default: '%g'
    cbar : bool, optional
      Display the colorbar. Default: True
    notext : bool, optional
      If True, no text is printed around the map
    norm : {'hist', 'log', None}
      Color normalization, hist= histogram equalized color mapping,
      log= logarithmic color mapping, default: None (linear color mapping)
    cmap : a color map
       The colormap to use (see matplotlib.cm)
    badcolor : str
      Color to use to plot bad values
    bgcolor : str
      Color to use for background
    hold : bool, optional
      If True, replace the current Axes by an Equidistant AzimuthalAxes.
      use this if you want to have multiple maps on the same
      figure. Default: False
    sub : int, scalar or sequence, optional
      Use only a zone of the current figure (same syntax as subplot).
      Default: None
    reuse_axes : bool, optional
      If True, reuse the current Axes (should be a AzimuthalAxes). This is
      useful if you want to overplot with a partially transparent colormap,
      such as for plotting a line integral convolution. Default: False
    margins : None or sequence, optional
      Either None, or a sequence (left,bottom,right,top)
      giving the margins on left,bottom,right and top
      of the axes. Values are relative to figure (0-1).
      Default: None
    return_projected_map : bool
      if True returns the projected map in a 2d numpy array
    alpha : float, array-like or None
      An array containing the alpha channel, supports masked maps, see the `ma` function.
      If None, no transparency will be applied.
    fontsize : dict, optional
      Override font sizes. Allowed keys are 'xlabel', 'ylabel', 'title', 'xtick_label', 
      'ytick_label', 'cbar_label', 'cbar_tick_label'. Default: None uses relative sizes
      that scale with DPI ('large' for most text). To exactly reproduce plots from 
      previous versions, use: ``fontsize={'cbar_label': 14}``

    See Also
    --------
    mollview, gnomview, cartview, orthview
    """
    # Create the figure
    import pylab

    if map is None:
        map = np.zeros(12) + np.inf
        cbar = False

    # Ensure that the nside is valid
    nside = pixelfunc.get_nside(map)
    pixelfunc.check_nside(nside, nest=nest)

    # Set up font sizes - use provided values or defaults
    if fontsize is None:
        fontsize = {}
    fontsize_defaults = {
        'xlabel': 'large',
        'ylabel': 'large', 
        'title': 'large',
        'xtick_label': 'large',
        'ytick_label': 'large',
        'cbar_label': 'large',
        'cbar_tick_label': 'large',
    }
    for key in fontsize_defaults:
        if key not in fontsize:
            fontsize[key] = fontsize_defaults[key]

    if not (hold or sub or reuse_axes):
        f = pylab.figure(fig, figsize=(8.5, 5.4))
        if not margins:
            margins = (0.02, 0.05, 0.02, 0.05)
        extent = (0.0, 0.0, 1.0, 1.0)
    elif hold:
        f = pylab.gcf()
        left, bottom, right, top = np.array(f.gca().get_position()).ravel()
        if not margins:
            margins = (0.0, 0.0, 0.0, 0.0)
        extent = (left, bottom, right - left, top - bottom)
        f.delaxes(f.gca())
    elif reuse_axes:
        f = pylab.gcf()
    else:  # using subplot syntax
        f = pylab.gcf()
        if hasattr(sub, "__len__"):
            nrows, ncols, idx = sub
        else:
            nrows, ncols, idx = sub // 100, (sub % 100) // 10, (sub % 10)
        if idx < 1 or idx > ncols * nrows:
            raise ValueError("Wrong values for sub: %d, %d, %d" % (nrows, ncols, idx))
        c, r = (idx - 1) % ncols, (idx - 1) // ncols
        if not margins:
            margins = (0.01, 0.0, 0.0, 0.02)
        extent = (
            c * 1.0 / ncols,
            1.0 - (r + 1) * 1.0 / nrows,
            1.0 / ncols,
            1.0 / nrows,
        )
    if not reuse_axes:
        extent = (
            extent[0] + margins[0],
            extent[1] + margins[1],
            extent[2] - margins[2] - margins[0],
            extent[3] - margins[3] - margins[1],
        )

    # Starting to draw : turn interactive off
    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    try:
        if reuse_axes:
            ax = f.gca()
        else:
            ax = PA.HpxAzimuthalAxes(
                f, extent, coord=coord, rot=rot, format=format, flipconv=flip
            )
            f.add_axes(ax)
        if remove_dip:
            map = pixelfunc.remove_dipole(
                map, gal_cut=gal_cut, nest=nest, copy=True
            )
        elif remove_mono:
            map = pixelfunc.remove_monopole(
                map, gal_cut=gal_cut, nest=nest, copy=True
            )
        img = ax.projmap(
            map,
            nest=nest,
            xsize=xsize,
            ysize=ysize,
            reso=reso,
            lamb=lamb,
            half_sky=half_sky,
            coord=coord,
            vmin=min,
            vmax=max,
            cmap=cmap,
            badcolor=badcolor,
            bgcolor=bgcolor,
            norm=norm,
            alpha=alpha,
        )
        if cbar:
            im = ax.get_images()[0]
            b = im.norm.inverse(np.linspace(0, 1, im.cmap.N + 1))
            v = np.linspace(im.norm.vmin, im.norm.vmax, im.cmap.N)
            mappable = plt.cm.ScalarMappable(
                norm=matplotlib.colors.Normalize(vmin=im.norm.vmin, vmax=im.norm.vmax),
                cmap=cmap,
            )
            if matplotlib.__version__ >= "0.91.0":
                cb = f.colorbar(
                    mappable,
                    ax=ax,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(),
                    pad=0.05,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            else:
                # for older matplotlib versions, no ax kwarg
                cb = f.colorbar(
                    mappable,
                    orientation="horizontal",
                    shrink=0.5,
                    aspect=25,
                    ticks=PA.BoundaryLocator(),
                    pad=0.05,
                    fraction=0.1,
                    boundaries=b,
                    values=v,
                    format=format,
                )
            cb.solids.set_rasterized(True)
        if title is None:
            if lamb:
                title = "Lambert azimuthal equal-area view"
            else:
                title = "Azimuthal equidistant view"
        ax.set_title(title)
        coordsys_label = ax.proj.coordsysstr
        if not notext and coordsys_label:
            ax.text(
                0.86,
                0.05,
                coordsys_label,
                fontsize=fontsize['cbar_label'],
                fontweight="bold",
                transform=ax.transAxes,
            )
        if cbar:
            cb.ax.text(
                0.5,
                -1.0,
                unit,
                fontsize=fontsize['cbar_label'],
                transform=cb.ax.transAxes,
                ha="center",
                va="center",
            )
        f.sca(ax)
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()
            # pylab.show()
    if return_projected_map:
        return img


def graticule(dpar=None, dmer=None, coord=None, local=None, fontsize=None, **kwds):
    """Draw a graticule on the current Axes.

    Parameters
    ----------
    dpar, dmer : float, scalars
      Interval in degrees between meridians and between parallels
    coord : {'E', 'G', 'C', None}
      The coordinate system of the graticule (make rotation if needed,
      using coordinate system of the map if it is defined), typically
      'E' for Ecliptic, 'G' for Galactic or 'C' for Celestial (equatorial).
    local : bool
      If True, draw a local graticule (no rotation is performed, useful for
      a gnomonic view, for example)
    fontsize : float or str, optional
      Font size for coordinate system label. Default: 'large' (scales with DPI).
      To exactly reproduce plots from previous versions, use: ``fontsize=14``

    Notes
    -----
    Other keyword parameters will be transmitted to the projplot function.

    See Also
    --------
    delgraticules
    """
    import pylab

    if fontsize is None:
        fontsize = 'large'

    f = pylab.gcf()
    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    try:
        if len(f.get_axes()) == 0:
            ax = PA.HpxMollweideAxes(f, (0.02, 0.05, 0.96, 0.9), coord=coord)
            f.add_axes(ax)
            coordsys_label = ax.proj.coordsysstr
            if coordsys_label:
                ax.text(
                    0.86,
                    0.05,
                    coordsys_label,
                    fontsize=fontsize,
                    fontweight="bold",
                    transform=ax.transAxes,
                )
        for ax in f.get_axes():
            if isinstance(ax, PA.SphericalProjAxes):
                ax.graticule(dpar=dpar, dmer=dmer, coord=coord, local=local, **kwds)
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()


def delgraticules():
    """Delete all graticules previously created on the Axes.

    See Also
    --------
    graticule
    """
    import pylab

    f = pylab.gcf()
    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    try:
        for ax in f.get_axes():
            if isinstance(ax, PA.SphericalProjAxes):
                ax.delgraticules()
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()
            # pylab.show()


def projplot(*args, **kwds):
    import pylab

    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    ret = None
    try:
        ax = pylab.gca()
        if isinstance(ax, PA.SphericalProjAxes):
            ret = ax.projplot(*args, **kwds)
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()
            # pylab.show()
    return ret


projplot.__doc__ = PA.SphericalProjAxes.projplot.__doc__


def projscatter(*args, **kwds):
    import pylab

    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    ret = None
    try:
        ax = pylab.gca()
        if isinstance(ax, PA.SphericalProjAxes):
            ret = ax.projscatter(*args, **kwds)
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()
            # pylab.show()
    return ret


projscatter.__doc__ = PA.SphericalProjAxes.projscatter.__doc__


def projtext(*args, **kwds):
    import pylab

    wasinteractive = pylab.isinteractive()
    pylab.ioff()
    ret = None
    try:
        ax = pylab.gca()
        if isinstance(ax, PA.SphericalProjAxes):
            ret = ax.projtext(*args, **kwds)
    finally:
        pylab.draw()
        if wasinteractive:
            pylab.ion()
            # pylab.show()
    return ret


projtext.__doc__ = PA.SphericalProjAxes.projtext.__doc__



"""Test legend functionality with newvisufunc

This test verifies that legends work correctly with newprojplot
when an explicit location is specified.
"""
import matplotlib
matplotlib.use('Agg')  # Use non-interactive backend for testing

import healpy as hp
import numpy as np
import matplotlib.pyplot as plt
import pytest
import tempfile
import os


def test_newprojplot_legend_with_explicit_loc():
    """Test that legend works when loc is explicitly specified"""
    # Create a simple map
    nside = 16
    npix = hp.nside2npix(nside)
    m = np.random.random(npix)
    
    # Create projection view
    hp.newvisufunc.projview(m, projection_type='mollweide')
    
    # Add some plot elements with labels
    theta1 = np.linspace(0, np.pi, 50)
    phi1 = np.zeros_like(theta1) + np.pi/2
    hp.newprojplot(theta1, phi1, 'r-', linewidth=2, label='Meridian')
    
    theta2 = np.zeros(50) + np.pi/2
    phi2 = np.linspace(-np.pi, np.pi, 50)
    hp.newprojplot(theta2, phi2, 'b--', linewidth=2, label='Equator')
    
    # Create legend with explicit location - this should work without hanging
    legend = plt.legend(loc='upper right')
    
    # Verify legend was created
    assert legend is not None
    assert len(legend.get_texts()) == 2
    
    # Save to temporary file
    with tempfile.NamedTemporaryFile(suffix='.png', delete=False) as f:
        temp_file = f.name
    # File is now closed, safe to write to it
    plt.savefig(temp_file, dpi=50, bbox_inches='tight')
    plt.close()
    
    # Clean up
    try:
        os.unlink(temp_file)
    except (OSError, FileNotFoundError):
        pass


def test_newprojplot_legend_with_bbox():
    """Test that legend works with bbox_to_anchor"""
    # Create a simple map
    nside = 16
    npix = hp.nside2npix(nside)
    m = np.random.random(npix)
    
    # Create projection view
    hp.newvisufunc.projview(m, projection_type='hammer')
    
    # Add plot element with label
    theta = np.linspace(0, np.pi, 100)
    phi = np.sin(theta) * np.pi
    hp.newprojplot(theta, phi, 'g-', linewidth=2, label='Sine curve')
    
    # Create legend with bbox_to_anchor - this should also work
    legend = plt.legend(bbox_to_anchor=(1.05, 1), loc='upper left')
    
    assert legend is not None
    
    # Save to temporary file
    with tempfile.NamedTemporaryFile(suffix='.png', delete=False) as f:
        temp_file = f.name
    # File is now closed, safe to write to it
    plt.savefig(temp_file, dpi=50, bbox_inches='tight')
    plt.close()
    
    # Clean up
    try:
        os.unlink(temp_file)
    except (OSError, FileNotFoundError):
        pass


def test_newprojplot_returns_line_objects():
    """Test that newprojplot returns proper Line2D objects"""
    nside = 16
    npix = hp.nside2npix(nside)
    m = np.random.random(npix)
    
    hp.newvisufunc.projview(m, projection_type='mollweide')
    
    # Call newprojplot and check return value
    theta = np.array([0.5, 1.0, 1.5])
    phi = np.array([0.0, 0.5, 1.0])
    lines = hp.newprojplot(theta, phi, 'ro-', label='Test')
    
    # Verify return type
    assert isinstance(lines, list)
    assert len(lines) == 1
    
    # Verify the line has the label
    assert lines[0].get_label() == 'Test'
    
    plt.close()


def test_issue_example_with_explicit_loc():
    """Test the example from the issue with explicit loc parameter
    
    This reproduces the issue scenario but with the fix applied.
    """
    # Reproduce the issue example
    hp.newvisufunc.projview(np.random.random(12*16**2))
    
    # Make a circle
    # Create 10000 points covering 360 degrees
    num_points = 10000
    step_size = 360.0 / num_points  # 0.036 degrees per step
    circle = np.zeros(num_points) + np.radians(45), np.arange(-180, 180, step_size)
    
    # Rotate it
    rotMat = [[0, 0, 1], [0, 1, 0], [-1, 0, 0]]
    rotCircle = hp.rotator.rotateDirection(rotMat, circle)
    
    # Add it to the plot
    hp.newprojplot(*rotCircle, linewidth=1, linestyle="", marker=".", label="a circle", c="r")
    
    # Add legend with explicit location - this should work
    legend = plt.legend(loc='upper right')
    
    assert legend is not None
    
    # Save to temporary file
    with tempfile.NamedTemporaryFile(suffix='.png', delete=False) as f:
        temp_file = f.name
    # File is now closed, safe to write to it
    plt.savefig(temp_file, dpi=50, bbox_inches='tight')
    plt.close()
    
    # Clean up
    try:
        os.unlink(temp_file)
    except (OSError, FileNotFoundError):
        pass



import numpy as np
import pytest

from healpy import pixelfunc as pf


def test_check_theta_valid_accepts_bounds():
    pf.check_theta_valid(0.0)
    pf.check_theta_valid(np.pi)
    pf.check_theta_valid(np.array([0.0, np.pi / 2]))


@pytest.mark.parametrize("theta", [-1e-6, np.pi + 2e-5])
def test_check_theta_valid_rejects_out_of_range(theta):
    with pytest.raises(ValueError):
        pf.check_theta_valid(theta)


def test_check_theta_valid_rejects_out_of_range_arrays():
    with pytest.raises(ValueError):
        pf.check_theta_valid(np.array([0.0, -1e-6]))
    with pytest.raises(ValueError):
        pf.check_theta_valid(np.array([np.pi, np.pi + 2e-5]))


def test_lonlat_thetaphi_roundtrip():
    lon = np.array([0.0, 45.0, 120.0])
    lat = np.array([-30.0, 0.0, 60.0])

    theta, phi = pf.lonlat2thetaphi(lon, lat)
    lon_back, lat_back = pf.thetaphi2lonlat(theta, phi)

    assert np.allclose(lon_back, lon)
    assert np.allclose(lat_back, lat)


def test_maptype_single_map():
    npix = pf.nside2npix(1)
    single_map = np.arange(npix, dtype=float)

    assert pf.maptype(single_map) == 0


def test_maptype_sequence_of_maps():
    npix = pf.nside2npix(1)
    stack = [np.arange(npix, dtype=float), np.zeros(npix)]

    assert pf.maptype(stack) == len(stack)


def test_maptype_errors_on_mismatch():
    with pytest.raises(TypeError):
        pf.maptype([np.arange(12), np.zeros(8)])


def test_maptype_errors_on_scalar():
    with pytest.raises(TypeError):
        pf.maptype(42)


def test_get_map_size_array_like():
    npix = pf.nside2npix(2)
    map_vector = np.arange(npix, dtype=float)

    assert pf.get_map_size(map_vector) == npix


def test_get_map_size_dict_like_with_nside():
    npix = pf.nside2npix(2)
    explicit = {0: 1.0, npix - 1: 2.0, "nside": 2}

    assert pf.get_map_size(explicit) == npix



import pytest
import numpy as np
from healpy import remove_monopole
from healpy.utils.deprecation import HealpyDeprecationWarning


def test_healpy_deprecation():
    with pytest.warns(HealpyDeprecationWarning):
        remove_monopole(np.zeros(12), verbose=True)



import importlib.util
from pathlib import Path
from unittest import mock

import pytest

setuptools = pytest.importorskip("setuptools")


def _load_setup_module():
    """Load setup.py without triggering a real setuptools.setup call."""
    setup_path = Path(__file__).resolve().parents[1] / "setup.py"
    spec = importlib.util.spec_from_file_location("healpy_setup_test", setup_path)
    module = importlib.util.module_from_spec(spec)
    with mock.patch("setuptools.setup"):
        spec.loader.exec_module(module)
    return module


def test_build_external_clib_missing_library(monkeypatch, tmp_path_factory):
    setup_mod = _load_setup_module()

    # Instantiate the command with writable build directories.
    dist = setuptools.dist.Distribution()
    cmd = setup_mod.build_external_clib(dist)
    cmd.build_temp = str(tmp_path_factory.mktemp("build_temp"))
    cmd.build_clib = str(tmp_path_factory.mktemp("build_clib"))

    def fake_check_output(cmd_args, env=None):
        cmd_list = list(cmd_args)
        if cmd_list[:2] == ["pkg-config", "--version"]:
            return b"1.8.0"
        raise setup_mod.CalledProcessError(1, cmd_args)

    monkeypatch.setattr(setup_mod, "check_output", fake_check_output)

    with pytest.raises(setup_mod.ExecError) as excinfo:
        cmd.build_library(
            "fake_lib",
            pkg_config_name="fake_lib",
            local_source=None,
        )

    assert str(excinfo.value) == "library 'fake_lib' is not installed"
